part of nueli.core.config;

MediaConfig get $media => MediaConfig();
MediaConfig get $icon => MediaConfig();

/// Icon config
class MediaConfig extends Config<MediaConfig, Media> {
  Color? _color;
  Alignment? _align;
  InkConfig? _ink;

  MediaConfig() : super(Media());
  MediaConfig.icon() : super(Media());

  MediaConfig _config({
    Color? color,
    Alignment? align,
    InkConfig? ink,
  }) {
    _color = color ?? _color;
    _align = align ?? _align;
    _ink = ink ?? _ink;
    return this;
  }

  MediaConfig color(Color color) => _config(color: color);
  MediaConfig get color_bg => color(palette.background); // ignore: non_constant_identifier_names
  MediaConfig get color_text => color(palette.text); // ignore: non_constant_identifier_names
  MediaConfig get color_hint => color(palette.text.withAlpha(150)); // ignore: non_constant_identifier_names
  MediaConfig get color_none => color(palette.none); // ignore: non_constant_identifier_names
  MediaConfig get plain => color_none;

  MediaConfig align(Alignment align) => _config(align: align);
  MediaConfig get align_start => align(Alignment.topLeft); // ignore: non_constant_identifier_names
  MediaConfig get align_center => align(Alignment.center); // ignore: non_constant_identifier_names
  MediaConfig get align_end => align(Alignment.bottomRight); // ignore: non_constant_identifier_names

  MediaConfig ink(InkConfig ink) => _config(ink: ink);

  MediaConfig withMedia(MediaData? media) => this & media;
  MediaConfig withSize(Size? size) => this & size;
  MediaConfig withBlendMode(BlendMode? blendMode) => this & blendMode;
  MediaConfig withBoxFit(BoxFit? boxFit) => this & boxFit;

  @override
  MediaConfig operator *(double factor) {
    final mediaAp = _MediaApprox(this);
    return _config(
      color: mediaAp.color * factor,
      align: mediaAp.align * factor,
    );
  }

  @override
  MediaConfig operator -(MediaConfig operand) {
    final mediaAp = _MediaApprox(this);
    final mediaAp2 = _MediaApprox(operand);
    return _config(
      align: mediaAp.align - mediaAp.align,
      color: mediaAp.color - mediaAp2.color,
    );
  }

  @override
  MediaConfig operator +(MediaConfig operand) {
    final mediaAp = _MediaApprox(this);
    final mediaAp2 = _MediaApprox(operand);
    return _config(
      align: mediaAp.align + mediaAp.align,
      color: mediaAp.color + mediaAp2.color,
    );
  }

  @override
  MediaConfig lerp(ico) {
    if (ico is MediaConfig)
      _config(
        align: ico._align ?? _align,
        color: ico._color ?? _color,
      );
    return super.lerp(ico);
  }

  MediaData get toMediaData => data.media ?? MediaData.data(CupertinoIcons.exclamationmark_circle);

  @override
  Widget call() {
    final mediaAp = _MediaApprox(this);

    /// library
    if (toMediaData.data != null) {
      final media = Icon(
        toMediaData.data,
        color: mediaAp.color,
        size: data.shortestSide,
      );
      if (_ink == null) return media;
      return (_ink! & $radii.all(data.longestSide) & media | InkType.response).call();
    }

    /// asset
    if (toMediaData.path != null) {
      final media = Image.asset(
        toMediaData.path!,
        fit: data.boxFit,
        alignment: mediaAp.align,
        color: mediaAp.color,
        colorBlendMode: data.blendMode,
        width: data.width,
        height: data.height,
      );
      if (_ink == null) return media;
      return (_ink! & $radii.all(data.longestSide) & media | InkType.response).call();
    }

    /// network
    if (toMediaData.src != null) {
      Widget loader(ImageChunkEvent chunk) {
        final loadValue = (chunk.expectedTotalBytes != null ? chunk.cumulativeBytesLoaded / chunk.expectedTotalBytes! : null);
        return Center(
          child: Container(
            height: 36,
            width: 36,
            padding: const EdgeInsets.all(8.0),
            child: CircularProgressIndicator(
              strokeWidth: 2,
              backgroundColor: palette.background,
              value: loadValue,
            ),
          ),
        );
      }

      Widget error([StackTrace? trace]) {
        final errType = (data.width ?? 0) > 100 && (data.height ?? 0) > 50;
        final errMedia = $media.color(palette.error) & CupertinoIcons.exclamationmark_circle;
        final errText = $txt.color((_color ?? palette.text).withAlpha(100)) & 'An error occurred' & TextAlign.center;
        return Center(child: Padding(padding: const EdgeInsets.all(8.0), child: errType ? errText() : errMedia()));
      }

      final media = Image.network(
        toMediaData.src!,
        color: mediaAp.color,
        fit: data.boxFit,
        alignment: mediaAp.align,
        colorBlendMode: data.blendMode,
        height: data.height,
        width: data.width,
        errorBuilder: (c, o, s) => error(s),
        loadingBuilder: (c, child, chunk) => chunk == null ? child : loader(chunk),
      );
      if (_ink == null) return media;
      return (_ink! & $radii.all(data.longestSide) & media | InkType.response).call();
    }

    return SizedBox();
  }
}

class Media extends ConfigData<Media> {
  MediaData? media;
  double? width;
  double? height;
  BlendMode? blendMode;
  BoxFit? boxFit;

  Media({
    Widget? child,
    this.media,
    this.width,
    this.height,
    this.blendMode,
    this.boxFit,
  }) : super(child);

  double get shortestSide => min(width ?? 0, height ?? 0);
  double get longestSide => max(width ?? 0, height ?? 0);

  Media lerp(data) {
    if (data is IconData) media = MediaData.data(data);
    if (data is MediaData) media = data;

    if (data is Size) {
      width = data.width;
      height = data.height;
    }

    if (data is BlendMode) blendMode = data;
    if (data is BoxFit) boxFit = data;

    if (data is Media) {
      media = data.media;
      width = data.width;
      height = data.height;
      blendMode = data.blendMode;
      boxFit = data.boxFit;
    }
    return super.lerp(data);
  }
}

class MediaTween extends Tween<MediaConfig> {
  MediaTween({MediaConfig? begin, MediaConfig? end}) : super(begin: begin, end: end);

  @override
  MediaConfig transform(double value) => (begin ?? $media) + ((end ?? $media) - (begin ?? $media)) * value;
}

class MediaTransition extends AnimatedWidget {
  final Widget? child;
  MediaTransition({required Animation<MediaConfig> media, this.child}) : super(listenable: media);
  Animation<MediaConfig> get media => listenable as Animation<MediaConfig>;

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: listenable,
      builder: (c, child) => media.value.withChild(child)(),
    );
  }
}

class _MediaApprox {
  final MediaConfig data;
  _MediaApprox(this.data);

  Color get color => data._color ?? palette.none;
  Alignment get align => data._align ?? Alignment.center;
}
